# Put this on module path in file newrelic_django_nap.py.
#
# In agent configuration file add:
#
# [import-hook:nap.publisher]
# enabled = true
# execute = nap.newrelic:instrument_django_nap_publisher
from __future__ import absolute_import

from newrelic.agent import (current_transaction, set_transaction_name,
    FunctionTrace, callable_name, ObjectWrapper)

def instrument_django_nap_publisher(module):

    def execute_wrapper(wrapped, instance, args, kwargs):

        transaction = current_transaction()

        if transaction is None:
            return wrapped(*args, **kwargs)

        def _handler(handler, *args, **kwargs):
            return handler

        handler = _handler(*args, **kwargs)

        name = callable_name(handler)

        set_transaction_name(name)

        with FunctionTrace(transaction, name):
            return wrapped(*args, **kwargs)

    module.BasePublisher.execute = ObjectWrapper(
        module.BasePublisher.execute, None, execute_wrapper)




import json


class Engine(object):
    # The list of content types we match
    CONTENT_TYPES = []
    def dumps(self, data): # pragma: no cover
        '''How to serialiser an object'''
        raise NotImplementedError
    
    def loads(self, data): # pragma: no cover
        '''How to deserialise a string'''
        raise NotImplementedError


class JsonEngine(Engine):
    CONTENT_TYPES = ['application/json',]
    def dumps(self, data):
        return json.dumps(data)
    def loads(self, data):
        return json.loads(data)


try:
    import msgpack
except ImportError:
    pass
else:
    class MsgPackEngine(Engine):
        CONTENT_TYPES = ['application/x-msgpack',]
        def dumps(self, data):
            return msgpack.dumps(data)
        def loads(self, data):
            return msgpack.loads(data)



from django.test import TestCase
from polls import models
from nap.models import ModelSerialiser
from nap.publisher import Publisher
from nap import api


class PollSerialiser(ModelSerialiser):
    class Meta:
        model = models.Poll


class PollPublisher(Publisher):
    serialiser = PollSerialiser()
    api_name = 'polls'

    def get_object_list(self):
        return models.Poll.objects.all()


class ApiTest(TestCase):

    def tearDown(self):
        try:
            del api.APIS['v1']
        except KeyError:
            pass

    def test_patterns(self):
        self.assertTrue(api.patterns() == [])

    def test_register(self):
        self.assertTrue(len(api.APIS.keys()) == 0)
        api.register('v1', PollPublisher)
        self.assertTrue(len(api.APIS.keys()) == 1)
        self.assertTrue(len(api.patterns()) == 1)



import os.path

INSTALLED_APPS = [
    'nap',
    'polls',
]

SITE_ID = 1

ROOT_URLCONF = "urls"

DEBUG = True

STATIC_URL = '/static/'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(os.path.dirname(__file__), 'database.db'),
    }
}




'''
Generic views for using Serialisers and Publishers
'''
from django.views import generic

class DetailView(generic.DetailView):
    def dispatch(self, request, **kwargs):
        self.publisher = self.publisher_class(request, **kwargs)
        return super(DetailView, self).dispatch(request, **kwargs)

    def get_object(self, queryset=None):
        return self.publisher.get_object(self.kwargs[self.pk_url_slug])

    def get_queryset(self):
        return self.publisher.get_object_list()

    def get_context_data(self, **kwargs):
        data = super(DetailView, self).get_context_data(**kwargs)
        page = self.publisher.
